@@ -24,14 +24,17 @@
  * The current implementation remembers all file names ever used.
  */
 
+#include <stdlib.h>
+#include <string.h>
 #include "vim.h"
 
 static void	enter_buffer __ARGS((BUF *));
 static char_u	*buflist_match __ARGS((vim_regexp *prog, BUF *buf));
 static char_u	*buflist_match_try __ARGS((vim_regexp *prog, char_u *name));
-static void	buflist_setlnum __ARGS((BUF *, linenr_t));
-static void	free_buf_winlnum __ARGS((BUF *buf));
+static void	buflist_setfpos __ARGS((BUF *, linenr_t, colnr_t));
+static FPOS	*buflist_findfpos __ARGS((BUF *buf));
 static int	append_arg_number __ARGS((char_u *, int, int));
+static void	free_buffer __ARGS((BUF *));
 
 /*
  * Open current buffer, that is: open the memfile and read the file into memory
@@ -114,9 +117,9 @@ open_buffer(read_stdin)
 		|| modified_was_set	/* ":set modified" used in autocmd */
 #endif
 		)
-	CHANGED;
+	changed();
     else if (retval != FAIL)
-	UNCHANGED(curbuf);
+	unchanged(curbuf, FALSE);
     curbuf->b_start_ffc = *curbuf->b_p_ff;    /* keep this fileformat */
 
     /* require "!" to overwrite the file, because it wasn't read completely */
@@ -124,7 +127,7 @@ open_buffer(read_stdin)
 	curbuf->b_notedited = TRUE;
 
 #ifdef AUTOCMD
-    apply_autocmds(EVENT_BUFENTER, NULL, NULL, FALSE);
+    apply_autocmds(EVENT_BUFENTER, NULL, NULL, FALSE, curbuf);
 #endif
 
     if (retval != FAIL)
@@ -182,7 +185,12 @@ close_buffer(win, buf, free_buf, del_buf)
     if (buf->b_nwindows > 0)
 	--buf->b_nwindows;
     if (buf->b_nwindows == 0 && win != NULL)
+    {
 	set_last_cursor(win);	/* may set b_last_cursor */
+	if (win == curwin && curwin->w_cursor.lnum != 1)
+				/* and remember last cursor position */
+	    buflist_setfpos(buf, curwin->w_cursor.lnum, curwin->w_cursor.col);
+    }
     if (buf->b_nwindows > 0 || !free_buf)
     {
 	if (buf == curbuf)
@@ -190,21 +198,23 @@ close_buffer(win, buf, free_buf, del_buf)
 	return;
     }
 
-    /*
-     * Free all things allocated for this buffer.
-     */
-    buf_freeall(buf);
+    /* Always remove the buffer when there is no file name. */
+    if (buf->b_ffname == NULL)
+	del_buf = TRUE;
 
     /*
-     * Remove the buffer from the list.  Always when there is no file name.
+     * Free all things allocated for this buffer.
+     * Also calls the "BufDelete" autocommands when del_buf is TRUE.
      */
-    if (buf->b_ffname == NULL || del_buf)
+    buf_freeall(buf, del_buf);
+    if (!buf_valid(buf))	    /* autocommands may delete the buffer */
+	return;
+
+    /*
+     * Remove the buffer from the list.
+     */
+    if (del_buf)
     {
-#ifdef AUTOCMD
-	apply_autocmds(EVENT_BUFDELETE, buf->b_ffname, buf->b_ffname, FALSE);
-	if (!buf_valid(buf))	    /* autocommands may delete the buffer */
-	    return;
-#endif
 	vim_free(buf->b_ffname);
 	vim_free(buf->b_sfname);
 	if (buf->b_prev == NULL)
@@ -215,9 +225,7 @@ close_buffer(win, buf, free_buf, del_buf)
 	    lastbuf = buf->b_prev;
 	else
 	    buf->b_next->b_prev = buf->b_prev;
-	free_buf_options(buf, TRUE);
-	free_buf_winlnum(buf);
-	vim_free(buf);
+	free_buffer(buf);
     }
     else
 	buf_clear(buf);
@@ -231,8 +239,7 @@ buf_clear(buf)
     BUF	    *buf;
 {
     buf->b_ml.ml_line_count = 1;
-    buf->b_changed = FALSE;
-    buf->b_start_ffc = *buf->b_p_ff;
+    unchanged(buf, TRUE);
 #ifndef SHORT_FNAME
     buf->b_shortname = FALSE;
 #endif
@@ -245,19 +252,23 @@ buf_clear(buf)
  * buf_freeall() - free all things allocated for the buffer
  */
     void
-buf_freeall(buf)
+buf_freeall(buf, del_buf)
     BUF	    *buf;
+    int	    del_buf;	/* buffer is going to be deleted */
 {
 #ifdef AUTOCMD
-    apply_autocmds(EVENT_BUFUNLOAD, buf->b_ffname, buf->b_ffname, FALSE);
+    apply_autocmds(EVENT_BUFUNLOAD, buf->b_fname, buf->b_fname, FALSE, buf);
     if (!buf_valid(buf))	    /* autocommands may delete the buffer */
 	return;
+    if (del_buf)
+    {
+	apply_autocmds(EVENT_BUFDELETE, buf->b_fname, buf->b_fname, FALSE, buf);
+	if (!buf_valid(buf))	    /* autocommands may delete the buffer */
+	    return;
+    }
 #endif
-#ifdef HAVE_PERL_INTERP
-    perl_buf_free(buf);
-#endif
-#ifdef HAVE_PYTHON
-    python_buffer_free(buf);
+#ifdef HAVE_TCL
+    tcl_buffer_free(buf);
 #endif
     u_blockfree(buf);		    /* free the memory allocated for undo */
     ml_close(buf, TRUE);	    /* close and delete the memline/memfile */
@@ -271,6 +282,34 @@ buf_freeall(buf)
 #endif
 }
 
+/*
+ * Free a buffer structure and some of the things it contains.
+ */
+    static void
+free_buffer(buf)
+    BUF		*buf;
+{
+    WINFPOS	*wlp;
+
+    /* Free the b_winfpos list for buffer "buf". */
+    while (buf->b_winfpos != NULL)
+    {
+	wlp = buf->b_winfpos;
+	buf->b_winfpos = wlp->wl_next;
+	vim_free(wlp);
+    }
+
+#ifdef HAVE_PERL_INTERP
+    perl_buf_free(buf);
+#endif
+#ifdef HAVE_PYTHON
+    python_buffer_free(buf);
+#endif
+
+    free_buf_options(buf, TRUE);
+    vim_free(buf);
+}
+
 /*
  * do_bufdel() - delete or unload buffer(s)
  *
@@ -442,6 +481,10 @@ do_buffer(action, start, dir, count, forceit)
 		if (buf == NULL)
 		    buf = lastbuf;
 	    }
+	    /* in non-help buffer, skip help buffers, and vv */
+	    if ((buf->b_help) !=
+			     (start == DOBUF_LAST ? lastbuf : curbuf)->b_help)
+		 count++;
 	}
     }
 
@@ -460,6 +503,10 @@ do_buffer(action, start, dir, count, forceit)
 	return FAIL;
     }
 
+#ifdef USE_GUI
+    need_mouse_correct = TRUE;
+#endif
+
     /*
      * delete buffer buf from memory and/or the list
      */
@@ -537,7 +584,8 @@ do_buffer(action, start, dir, count, forceit)
 		forward = FALSE;
 		continue;
 	    }
-	    if (buf->b_ml.ml_mfp != NULL)
+	    /* in non-help buffer, try to skip help buffers, and vv */
+	    if (buf->b_ml.ml_mfp != NULL && buf->b_help == curbuf->b_help)
 		break;
 	    if (forward)
 		buf = buf->b_next;
@@ -577,20 +625,20 @@ do_buffer(action, start, dir, count, forceit)
 
     setpcmark();
     curwin->w_alt_fnum = curbuf->b_fnum; /* remember alternate file */
-    buflist_altlnum();			 /* remember curpos.lnum */
+    buflist_altfpos();			 /* remember curpos */
 
     /* close_windows() or apply_autocmds() may change curbuf */
     delbuf = curbuf;
 
 #ifdef AUTOCMD
-    apply_autocmds(EVENT_BUFLEAVE, NULL, NULL, FALSE);
+    apply_autocmds(EVENT_BUFLEAVE, NULL, NULL, FALSE, curbuf);
     if (buf_valid(delbuf))
 #endif
     {
 	if (action == DOBUF_UNLOAD || action == DOBUF_DEL)
 	    close_windows(delbuf);
 	if (buf_valid(delbuf))
-	    close_buffer(NULL, delbuf,
+	    close_buffer(delbuf == curwin->w_buffer ? curwin : NULL, delbuf,
 		    (action == DOBUF_GOTO && !p_hid && !buf_changed(delbuf))
 			     || action == DOBUF_UNLOAD || action == DOBUF_DEL,
 		      action == DOBUF_DEL);
@@ -621,10 +669,11 @@ enter_buffer(buf)
 	need_fileinfo = TRUE;		/* display file info after redraw */
 	buf_check_timestamp(curbuf);	/* check if file has changed */
 #ifdef AUTOCMD
-	apply_autocmds(EVENT_BUFENTER, NULL, NULL, FALSE);
+	apply_autocmds(EVENT_BUFENTER, NULL, NULL, FALSE, curbuf);
 #endif
     }
-    buflist_getlnum();			/* restore curpos.lnum */
+    buflist_getfpos();			/* restore curpos.lnum and possibly
+					 * curpos.col */
     check_arg_idx();			/* check for valid arg_idx */
     maketitle();
     scroll_cursor_halfway(FALSE);	/* redisplay at correct position */
@@ -662,7 +711,7 @@ buflist_new(ffname, sfname, lnum, use_curbuf)
     {
 	vim_free(ffname);
 	if (lnum != 0)
-	    buflist_setlnum(buf, lnum);
+	    buflist_setfpos(buf, lnum, (colnr_t)0);
 	/* copy the options now, if 'cpo' doesn't have 's' and not done
 	 * already */
 	buf_copy_options(curbuf, buf, 0);
@@ -678,7 +727,13 @@ buflist_new(ffname, sfname, lnum, use_curbuf)
     if (use_curbuf && curbuf != NULL && curbuf->b_ffname == NULL &&
 		curbuf->b_nwindows <= 1 &&
 		(curbuf->b_ml.ml_mfp == NULL || bufempty()))
+    {
 	buf = curbuf;
+#ifdef AUTOCMD
+	/* It's like this buffer is deleted. */
+	apply_autocmds(EVENT_BUFDELETE, NULL, NULL, FALSE, curbuf);
+#endif
+    }
     else
     {
 	buf = (BUF *)alloc_clear((unsigned)sizeof(BUF));
@@ -694,27 +749,23 @@ buflist_new(ffname, sfname, lnum, use_curbuf)
 	buf->b_ffname = ffname;
 	buf->b_sfname = vim_strsave(sfname);
     }
-    if (buf->b_winlnum == NULL)
-	buf->b_winlnum = (WINLNUM *)alloc((unsigned)sizeof(WINLNUM));
+    if (buf->b_winfpos == NULL)
+	buf->b_winfpos = (WINFPOS *)alloc((unsigned)sizeof(WINFPOS));
     if ((ffname != NULL && (buf->b_ffname == NULL ||
-			 buf->b_sfname == NULL)) || buf->b_winlnum == NULL)
+			 buf->b_sfname == NULL)) || buf->b_winfpos == NULL)
     {
 	vim_free(buf->b_ffname);
 	buf->b_ffname = NULL;
 	vim_free(buf->b_sfname);
 	buf->b_sfname = NULL;
 	if (buf != curbuf)
-	{
-	    vim_free(buf->b_winlnum);
-	    free_buf_options(buf, TRUE);
-	    vim_free(buf);
-	}
+	    free_buffer(buf);
 	return NULL;
     }
 
     if (buf == curbuf)
     {
-	buf_freeall(buf);	/* free all things allocated for this buffer */
+	buf_freeall(buf, FALSE); /* free all things allocated for this buffer */
 	buf->b_nwindows = 0;
     }
     else
@@ -743,10 +794,11 @@ buflist_new(ffname, sfname, lnum, use_curbuf)
 	    top_file_num = 1;
 	}
 
-	buf->b_winlnum->wl_lnum = lnum;
-	buf->b_winlnum->wl_next = NULL;
-	buf->b_winlnum->wl_prev = NULL;
-	buf->b_winlnum->wl_win = curwin;
+	buf->b_winfpos->wl_fpos.lnum = lnum;
+	buf->b_winfpos->wl_fpos.col = 0;
+	buf->b_winfpos->wl_next = NULL;
+	buf->b_winfpos->wl_prev = NULL;
+	buf->b_winfpos->wl_win = curwin;
 
 #ifdef WANT_EVAL
 	var_init(&buf->b_vars);	    /* init internal variables */
@@ -786,11 +838,20 @@ free_buf_options(buf, free_p_ff)
     int	    free_p_ff;	    /* also free 'fileformat'? */
 {
     if (free_p_ff)
+    {
 	free_string_option(buf->b_p_ff);
+#ifdef MULTI_BYTE
+	free_string_option(buf->b_p_fe);
+#endif
+    }
+    free_string_option(buf->b_p_mps);
     free_string_option(buf->b_p_fo);
     free_string_option(buf->b_p_isk);
     free_string_option(buf->b_p_com);
     free_string_option(buf->b_p_nf);
+#ifdef SYNTAX_HL
+    free_string_option(buf->b_p_syn);
+#endif
 #ifdef CINDENT
     free_string_option(buf->b_p_cink);
     free_string_option(buf->b_p_cino);
@@ -805,7 +866,8 @@ free_buf_options(buf, free_p_ff)
 
 /*
  * get alternate file n
- * set linenr to lnum or altlnum if lnum == 0
+ * set linenr to lnum or altfpos.lnum if lnum == 0
+ *	also set cursor column to altfpos.col if 'startofline' is not set.
  * if (options & GETF_SETMARK) call setpcmark()
  * if (options & GETF_ALT) we are jumping to an alternate file.
  *
@@ -819,6 +881,8 @@ buflist_getfile(n, lnum, options, forceit)
     int		forceit;
 {
     BUF	    *buf;
+    FPOS    *fpos;
+    colnr_t col;
 
     buf = buflist_findnr(n);
     if (buf == NULL)
@@ -834,14 +898,28 @@ buflist_getfile(n, lnum, options, forceit)
     if (buf == curbuf)
 	return OK;
 
-    /* altlnum may be changed by getfile(), get it now */
+    /* altfpos may be changed by getfile(), get it now */
     if (lnum == 0)
-	lnum = buflist_findlnum(buf);
+    {
+	fpos = buflist_findfpos(buf);
+	lnum = fpos->lnum;
+	col = fpos->col;
+    }
+    else
+	col = 0;
+
     ++RedrawingDisabled;
     if (getfile(buf->b_fnum, NULL, NULL, (options & GETF_SETMARK),
 							  lnum, forceit) <= 0)
     {
 	--RedrawingDisabled;
+
+	/* cursor is at to BOL and w_cursor.lnum is checked due to getfile() */
+	if (!p_sol && col != 0)
+	{
+	    curwin->w_cursor.col = col;
+	    check_cursor_col();
+	}
 	return OK;
     }
     --RedrawingDisabled;
@@ -852,15 +930,22 @@ buflist_getfile(n, lnum, options, forceit)
  * go to the last know line number for the current buffer
  */
     void
-buflist_getlnum()
+buflist_getfpos()
 {
-    linenr_t	lnum;
+    FPOS	*fpos;
 
-    curwin->w_cursor.lnum = 1;
-    curwin->w_cursor.col = 0;
-    lnum = buflist_findlnum(curbuf);
-    if (lnum != 0 && lnum <= curbuf->b_ml.ml_line_count)
-	curwin->w_cursor.lnum = lnum;
+    fpos = buflist_findfpos(curbuf);
+
+    curwin->w_cursor.lnum = fpos->lnum;
+    check_cursor_lnum();
+
+    if (p_sol)
+	curwin->w_cursor.col = 0;
+    else
+    {
+	curwin->w_cursor.col = fpos->col;
+	check_cursor_col();
+    }
 }
 
 /*
@@ -1128,21 +1213,22 @@ buflist_nr2name(n, fullname, helptail)
 }
 
 /*
- * set the lnum for the buffer 'buf' and the current window
+ * set the lnum and col for the buffer 'buf' and the current window
  */
     static void
-buflist_setlnum(buf, lnum)
+buflist_setfpos(buf, lnum, col)
     BUF		*buf;
     linenr_t	lnum;
+    colnr_t	col;
 {
-    WINLNUM	*wlp;
+    WINFPOS	*wlp;
 
-    for (wlp = buf->b_winlnum; wlp != NULL; wlp = wlp->wl_next)
+    for (wlp = buf->b_winfpos; wlp != NULL; wlp = wlp->wl_next)
 	if (wlp->wl_win == curwin)
 	    break;
     if (wlp == NULL)		/* make new entry */
     {
-	wlp = (WINLNUM *)alloc((unsigned)sizeof(WINLNUM));
+	wlp = (WINFPOS *)alloc((unsigned)sizeof(WINFPOS));
 	if (wlp == NULL)
 	    return;
 	wlp->wl_win = curwin;
@@ -1152,16 +1238,16 @@ buflist_setlnum(buf, lnum)
 	if (wlp->wl_prev)
 	    wlp->wl_prev->wl_next = wlp->wl_next;
 	else
-	    buf->b_winlnum = wlp->wl_next;
+	    buf->b_winfpos = wlp->wl_next;
 	if (wlp->wl_next)
 	    wlp->wl_next->wl_prev = wlp->wl_prev;
     }
-    wlp->wl_lnum = lnum;
-/*
- * insert entry in front of the list
- */
-    wlp->wl_next = buf->b_winlnum;
-    buf->b_winlnum = wlp;
+    wlp->wl_fpos.lnum = lnum;
+    wlp->wl_fpos.col = col;
+
+    /* insert entry in front of the list */
+    wlp->wl_next = buf->b_winfpos;
+    buf->b_winfpos = wlp;
     wlp->wl_prev = NULL;
     if (wlp->wl_next)
 	wlp->wl_next->wl_prev = wlp;
@@ -1169,6 +1255,30 @@ buflist_setlnum(buf, lnum)
     return;
 }
 
+/*
+ * find the position (lnum and col) for the buffer 'buf' for the current window
+ * returns a pointer to no_position if no position is found
+ */
+    static FPOS *
+buflist_findfpos(buf)
+    BUF		*buf;
+{
+    WINFPOS	*wlp;
+    static	FPOS no_position = {1, 0};
+
+    for (wlp = buf->b_winfpos; wlp != NULL; wlp = wlp->wl_next)
+	if (wlp->wl_win == curwin)
+	    break;
+
+    if (wlp == NULL)	/* if no fpos for curwin, use the first in the list */
+	wlp = buf->b_winfpos;
+
+    if (wlp != NULL)
+	return &(wlp->wl_fpos);
+    else
+	return &no_position;
+}
+
 /*
  * find the lnum for the buffer 'buf' for the current window
  */
@@ -1176,36 +1286,7 @@ buflist_setlnum(buf, lnum)
 buflist_findlnum(buf)
     BUF	    *buf;
 {
-    WINLNUM	*wlp;
-
-    for (wlp = buf->b_winlnum; wlp != NULL; wlp = wlp->wl_next)
-	if (wlp->wl_win == curwin)
-	    break;
-
-    if (wlp == NULL)	/* if no lnum for curwin, use the first in the list */
-	wlp = buf->b_winlnum;
-
-    if (wlp)
-	return wlp->wl_lnum;
-    else
-	return (linenr_t)1;
-}
-
-/*
- * Free the b_winlnum list for buffer "buf".
- */
-    static void
-free_buf_winlnum(buf)
-    BUF	    *buf;
-{
-    WINLNUM	*wlp;
-
-    while (buf->b_winlnum != NULL)
-    {
-	wlp = buf->b_winlnum;
-	buf->b_winlnum = wlp->wl_next;
-	vim_free(wlp);
-    }
+    return buflist_findfpos(buf)->lnum;
 }
 
 /*
@@ -1223,7 +1304,7 @@ buflist_list()
 	if (buf->b_fname == NULL)
 	    STRCPY(NameBuff, "No File");
 	else
-	    home_replace(buf, buf->b_fname, NameBuff, MAXPATHL);
+	    home_replace(buf, buf->b_fname, NameBuff, MAXPATHL, TRUE);
 
 	sprintf((char *)IObuff, "%3d %c%c%c \"",
 		buf->b_fnum,
@@ -1378,17 +1459,19 @@ setaltfname(ffname, sfname, lnum)
 
 /*
  * Get alternate file name for current window.
- * Return NULL if there isn't any, and give error message.
+ * Return NULL if there isn't any, and give error message if requested.
  */
     char_u  *
-getaltfname()
+getaltfname(errmsg)
+    int		errmsg;		/* give error message */
 {
     char_u	*fname;
     linenr_t	dummy;
 
     if (buflist_name_nr(0, &fname, &dummy) == FAIL)
     {
-	emsg(e_noalt);
+	if (errmsg)
+	    emsg(e_noalt);
 	return NULL;
     }
     return fname;
@@ -1412,12 +1495,12 @@ buflist_add(fname)
 }
 
 /*
- * set alternate lnum for current window
+ * set alternate cursor position for current window
  */
     void
-buflist_altlnum()
+buflist_altfpos()
 {
-    buflist_setlnum(curbuf, curwin->w_cursor.lnum);
+    buflist_setfpos(curbuf, curwin->w_cursor.lnum, curwin->w_cursor.col);
 }
 
 /*
@@ -1481,7 +1564,7 @@ fileinfo(fullname, shorthelp, dont_truncate)
 	else
 	    name = curbuf->b_ffname;
 	home_replace(shorthelp ? curbuf : NULL, name, p,
-						(int)(IOSIZE - (p - buffer)));
+					  (int)(IOSIZE - (p - buffer)), TRUE);
     }
 
     sprintf((char *)buffer + STRLEN(buffer),
@@ -1524,7 +1607,7 @@ fileinfo(fullname, shorthelp, dont_truncate)
     if (dont_truncate)
 	msg(buffer);
     else
-	msg_trunc(buffer);
+	msg_trunc_attr(buffer, FALSE, 0);
 
     vim_free(buffer);
 }
@@ -1618,7 +1701,7 @@ maketitle()
 	if (buf == NULL)
 	    return;
 	STRCPY(buf, "VIM - ");
-	home_replace(curbuf, curbuf->b_ffname, buf + 6, IOSIZE - 6);
+	home_replace(curbuf, curbuf->b_ffname, buf + 6, IOSIZE - 6, TRUE);
 	append_arg_number(buf, FALSE, IOSIZE);
 	if (p_titlelen > 0)
 	{
@@ -1628,7 +1711,7 @@ maketitle()
 	    len = STRLEN(buf);
 	    if (len > maxlen)
 	    {
-		vim_memmove(buf + 6, buf + 6 + len - maxlen,
+		mch_memmove(buf + 6, buf + 6 + len - maxlen,
 							  (size_t)maxlen - 5);
 		buf[5] = '<';
 	    }
@@ -1637,33 +1720,36 @@ maketitle()
 	i_name = gettail(curbuf->b_ffname); /* use file name only for icon */
     }
 
-    if (*p_titlestring)
-	t_name = p_titlestring;
-    if (*p_iconstring)
-	i_name = p_iconstring;
-
     vim_free(lasttitle);
-    if (p_title && (lasttitle = alloc((unsigned)(vim_strsize(t_name) + 1)))
-								      != NULL)
+    lasttitle = NULL;
+    if (p_title)
     {
-	*lasttitle = NUL;
-	while (*t_name)
-	    STRCAT(lasttitle, transchar(*t_name++));
+	if (*p_titlestring)
+	    lasttitle = vim_strsave(p_titlestring);
+	else if ((lasttitle = alloc((unsigned)(vim_strsize(t_name) + 1)))
+								      != NULL)
+	{
+	    *lasttitle = NUL;
+	    while (*t_name)
+		STRCAT(lasttitle, transchar(*t_name++));
+	}
     }
-    else
-	lasttitle = NULL;
     vim_free(buf);
 
     vim_free(lasticon);
-    if (p_icon && (lasticon = alloc((unsigned)(vim_strsize(i_name) + 1)))
-								      != NULL)
+    lasticon = NULL;
+    if (p_icon)
     {
-	*lasticon = NUL;
-	while (*i_name)
-	    STRCAT(lasticon, transchar(*i_name++));
+	if (*p_iconstring)
+	    lasticon = vim_strsave(p_iconstring);
+	else if ((lasticon = alloc((unsigned)(vim_strsize(i_name) + 1)))
+								      != NULL)
+	{
+	    *lasticon = NUL;
+	    while (*i_name)
+		STRCAT(lasticon, transchar(*i_name++));
+	}
     }
-    else
-	lasticon = NULL;
 
     resettitle();
 }
@@ -1787,6 +1873,10 @@ do_arg_all(count, forceit)
     if (opened == NULL)
 	return;
 
+#ifdef USE_GUI
+    need_mouse_correct = TRUE;
+#endif
+
     /*
      * Try closing all windows that are not in the argument list.
      * When 'hidden' or "forceit" set the buffer becomes hidden.
@@ -1941,6 +2031,10 @@ do_buffer_all(count, all)
 
     setpcmark();
 
+#ifdef USE_GUI
+    need_mouse_correct = TRUE;
+#endif
+
     /*
      * Close superfluous windows (two windows for the same buffer).
      */
@@ -2200,7 +2294,7 @@ read_viminfo_bufferlist(line, fp, writing)
 	{
 	    buf->b_last_cursor.lnum = lnum;
 	    buf->b_last_cursor.col = col;
-	    buflist_setlnum(buf, lnum);
+	    buflist_setfpos(buf, lnum, col);
 	}
     }
 
@@ -2223,10 +2317,10 @@ write_viminfo_bufferlist(fp)
     fprintf(fp, "\n# Buffer list:\n");
     for (buf = firstbuf; buf != NULL ; buf = buf->b_next)
     {
-	if (buf->b_fname == NULL || buf->b_help)
+	if (buf->b_fname == NULL || buf->b_help || removable(buf->b_ffname))
 	    continue;
 
-	home_replace(NULL, buf->b_ffname, NameBuff, MAXPATHL);
+	home_replace(NULL, buf->b_ffname, NameBuff, MAXPATHL, TRUE);
 
 	fprintf(fp, "%%%s\t%ld\t%d\n", NameBuff,
 			(long)buf->b_last_cursor.lnum,
